/**
 * 扩展 legoflow-engine webpack 配置 vue 项目
 */

const { getSourceMap } = require('legoflow-engine/core/common/util')
const {
  babelLoaderOptions,
  cssLoaderOptions,
  postcssLoaderOptions,
  sassLoaderOptions,
  sassResourcesOptions
} = require('legoflow-engine/core/webpack-config/options.js')()

module.exports = ({ webpackConfig }) => {
  const sourceMap = !!getSourceMap()
  const vueStyleLoaderOptions = { sourceMap }

  // vue jsx
  babelLoaderOptions.plugins.unshift('babel-plugin-transform-vue-jsx')
  webpackConfig.module.rule('js').uses.get('babel-loader').options(babelLoaderOptions)

  // rule - vue
  webpackConfig
    .module
    .rule('vue')
    .test(/\.vue$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('vue-loader').loader('vue-loader').options({
      compilerOptions: {
        preserveWhitespace: false
      }
    }).end()

  // vue-css
  webpackConfig
    .module
    .rule('css')
    .test(/\.css$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('vue-css')
    .resourceQuery(/^\?vue/)
    .use('vue-style-loader').loader('vue-style-loader').options(vueStyleLoaderOptions).end()
    .use('css-loader').loader('css-loader').options(cssLoaderOptions).end()
    .use('postcss-loader').loader('postcss-loader').options(postcssLoaderOptions).end()
    .end()

  // vue-scss
  webpackConfig
    .module
    .rule('sass')
    .test(/\.s(a|c)ss$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('vue-sass')
    .resourceQuery(/^\?vue/)
    .use('vue-style-loader').loader('vue-style-loader').options(vueStyleLoaderOptions).end()
    .use('css-loader').loader('css-loader').options(cssLoaderOptions).end()
    .use('postcss-loader').loader('postcss-loader').options(postcssLoaderOptions).end()
    .use('sass-loader').loader('sass-loader').options(sassLoaderOptions).end()
    .when(sassResourcesOptions, config => config.use('sass-resources-loader').loader('sass-resources-loader').options(sassResourcesOptions).end())
    .end()

  // vue plugin
  webpackConfig
    .plugin('vue-loader')
    .use(require('vue-loader/lib/plugin'))
}
